package ch.unisg.tapas;

import ch.unisg.tapas.auctionhouse.adapter.common.clients.TapasMqttClient;
import ch.unisg.tapas.auctionhouse.adapter.in.messaging.mqtt.AuctionEventsMqttDispatcher;
import ch.unisg.tapas.auctionhouse.adapter.common.clients.WebSubSubscriber;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.core.env.ConfigurableEnvironment;

import java.net.URI;
import java.net.URISyntaxException;

/**
 * Main TAPAS Auction House application.
 */
@SpringBootApplication
public class TapasAuctionHouseApplication {
    private static final Logger LOGGER = LogManager.getLogger(TapasAuctionHouseApplication.class);

    public static String DEFAULT_MQTT_BROKER = "tcp://broker.hivemq.com:1883";

    private static ConfigurableEnvironment ENVIRONMENT;

    public static void main(String[] args) {
		SpringApplication tapasAuctioneerApp = new SpringApplication(TapasAuctionHouseApplication.class);
        ENVIRONMENT = tapasAuctioneerApp.run(args).getEnvironment();
        // TODO Set start up of message services with config
		// We will use these bootstrap methods in Week 6:
        // bootstrapMarketplaceWithWebSub();
        bootstrapMarketplaceWithMqtt();
	}
    /**
     * Discovers auction houses and subscribes to WebSub notifications
     */
	private static void bootstrapMarketplaceWithWebSub() {
        // discoverAuctionHouseEndpoints();

        WebSubSubscriber subscriber = new WebSubSubscriber(ENVIRONMENT.getProperty("auction.house.uri"));

        // for (AuctionHouseDiscoveryInformation endpoint : AuctionHouseDiscovery.getInstance().getAuctionHouseDiscoveryList()) {
        //     subscriber.subscribeToAuctionHouseEndpoint(endpoint.getWebSubUri().getValue());
        // }
        try {
            subscriber.subscribeToAuctionHouseEndpoint(new URI("http://6b4e-130-82-250-227.ngrok.io/websub-discovery"));
        } catch (URISyntaxException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * Connects to an MQTT broker, presumably the one used by all TAPAS groups to communicate with
     * one another
     */
    private static void bootstrapMarketplaceWithMqtt() {
        try {
            String broker = ENVIRONMENT.getProperty("mqtt.broker.uri");
            LOGGER.info("Bootstrapping Mqtt");
            if (broker == null) {
                broker = DEFAULT_MQTT_BROKER;
                LOGGER.info("No MQTT broker was set in application.propreties, going with default: "
                    + DEFAULT_MQTT_BROKER);
            }

            AuctionEventsMqttDispatcher dispatcher = new AuctionEventsMqttDispatcher();
            TapasMqttClient client = TapasMqttClient.getInstance(broker, dispatcher);
            client.startReceivingMessages();
        } catch (MqttException e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

	// private static void discoverAuctionHouseEndpoints() {

    //     AuctionHouseResourceDirectory rd = new AuctionHouseResourceDirectory(
    //         URI.create(ENVIRONMENT.getProperty("discovery.endpoint.uri"))
    //     );

    //     AuctionHouseDiscovery.getInstance().addAuctionHouseDiscoveryInformation(rd.retrieveAuctionHouseEndpoints());

    //     ScheduledExecutorService executor = Executors.newScheduledThreadPool(1);
    //     executor.scheduleAtFixedRate(crawlerRunnable, 300, 300, TimeUnit.SECONDS);
    // }


    // private static Runnable crawlerRunnable = new Runnable() {
    //     public void run() {
    //         AuctionHouseResourceDirectory rd = new AuctionHouseResourceDirectory(
    //             URI.create(ENVIRONMENT.getProperty("discovery.endpoint.uri"))
    //         );

	//         AuctionHouseDiscovery.getInstance().addAuctionHouseDiscoveryInformation(rd.retrieveAuctionHouseEndpoints());
    //     }
    // };
}
